----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:51:17 04/29/2010 
-- Design Name: 
-- Module Name:    FSM_SLAVE - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity FSM_SLAVE is
    Port ( rst : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           st : in  STD_LOGIC;
			  sel_1_s : out STD_LOGIC;
			  iter_id : out STD_LOGIC_VECTOR(3 downto 0);
			  arg_id : out STD_LOGIC_VECTOR(3 downto 0);
			  loop_ov : out STD_LOGIC;
			  vld_o : out STD_LOGIC);
end FSM_SLAVE;

architecture Behavioral of FSM_SLAVE is

TYPE state is (rst_st,
					idle_st,
					iter_1,
					iter_s);

signal cur_st,nxt_st: state;

component COUNTER_4b is
    Port ( rst : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           en : in  STD_LOGIC;
           val : inout  STD_LOGIC_VECTOR (3 downto 0));
end component;

signal   rst_lc, rst_lc_or, rst_iter : STD_LOGIC;
signal   en_lc : STD_LOGIC;
signal   lc_val,iter_val : STD_LOGIC_VECTOR(3 downto 0);
signal   iter_e, fiter_e, iter_cmp : STD_LOGIC;

component comp_4b is
  port (
    qa_eq_b : out STD_LOGIC; 
    clk : in STD_LOGIC := 'X'; 
    a : in STD_LOGIC_VECTOR ( 3 downto 0 ); 
    b : in STD_LOGIC_VECTOR ( 3 downto 0 ) 
  );
end component;

component shift_reg_1b_10c is
  port (
    clk : in STD_LOGIC := 'X'; 
    d : in STD_LOGIC_VECTOR ( 0 downto 0 ); 
    q : out STD_LOGIC_VECTOR ( 0 downto 0 ) 
  );
end component;

signal iter_cmp_v,iter_cmp_v_o: std_logic_vector(0 downto 0);

begin

process(clk)
begin
if (rst='1') then
	cur_st<= rst_st;
elsif (clk'event and clk='1') then
	cur_st <= nxt_st;
end if;
end process;

process(cur_st,st,iter_e)
begin 
case cur_st is

when rst_st =>

	rst_lc<='1';
	rst_iter<='1';	
	en_lc<='0';	
	
	sel_1_s<='0';
	loop_ov<='0';
			  
	nxt_st<=idle_st;
	
when idle_st =>

	rst_iter<='0';	
	en_lc<='0';	
	
	sel_1_s<='0';
	loop_ov<='0';	
	
	if st='1' then
		rst_lc<='1';
		nxt_st<=iter_1;
	else
		rst_lc<='0';
		nxt_st<=idle_st;
	end if;
	
when iter_1 =>

	rst_lc<='0';	
	rst_iter<='0';
	en_lc<='1';	
	
	sel_1_s<='0';
	loop_ov<='0';
	
	if iter_e='1' then
		nxt_st<=iter_s;
	else	
		nxt_st<=iter_1;
	end if;
	
when iter_s =>

	rst_lc<='0';
	en_lc<='1';	
	
	sel_1_s<='1';	
	
	if fiter_e='1' then
		rst_iter<='1';
		loop_ov<='1';
		nxt_st<=rst_st;
	else
		rst_iter<='0';
		loop_ov<='0';
		nxt_st<=iter_s;
	end if;

end case;
end process;

rst_lc_or<=rst_lc or iter_e;

LT_COUNTER: COUNTER_4b Port Map
( rst => rst_lc_or,
  clk => clk,
  en => en_lc,
  val => lc_val
);

arg_id<=lc_val;

LT_COMP: comp_4b Port Map
( qa_eq_b => iter_e, 
  clk => clk, 
  a => lc_val, 
  b => "1010" 
);

ITER_COUNTER: COUNTER_4b Port Map
( rst => rst_iter,
  clk => clk,
  en => iter_e,
  val => iter_val
);
iter_id<=iter_val;

ITER_COMP: comp_4b Port Map
( qa_eq_b => iter_cmp, 
  clk => clk, 
  a => iter_val, 
  b => "1011" 
);

fiter_e<= iter_cmp and iter_e;
iter_cmp_v(0)<=iter_cmp;

SHFT: shift_reg_1b_10c Port Map
(
    clk => clk,
    d => iter_cmp_v,
    q => iter_cmp_v_o
  );
vld_o<=iter_cmp_v_o(0);

end Behavioral;

